@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,6 +30,7 @@
 macro(AbsD)
 macro(AbsF)
 macro(AbsI)
+macro(AbsL)
 macro(AddD)
 macro(AddF)
 macro(AddI)
@@ -57,7 +58,6 @@ macro(CallLeafNoFP)
 macro(CallRuntime)
 macro(CallStaticJava)
 macro(CastII)
-macro(CastLL)
 macro(CastX2P)
 macro(CastP2X)
 macro(CastPP)
@@ -202,6 +202,8 @@ macro(Loop)
 macro(LoopLimit)
 macro(Mach)
 macro(MachProj)
+macro(MaxD)
+macro(MaxF)
 macro(MaxI)
 macro(MemBarAcquire)
 macro(LoadFence)
@@ -214,6 +216,8 @@ macro(MemBarReleaseLock)
 macro(MemBarVolatile)
 macro(MemBarStoreStore)
 macro(MergeMem)
+macro(MinD)
+macro(MinF)
 macro(MinI)
 macro(ModD)
 macro(ModF)
@@ -234,6 +238,8 @@ macro(NegF)
 macro(NeverBranch)
 macro(OnSpinWait)
 macro(Opaque1)
+macro(OpaqueLoopInit)
+macro(OpaqueLoopStride)
 macro(Opaque2)
 macro(Opaque3)
 macro(Opaque4)
@@ -265,6 +271,16 @@ macro(RoundDouble)
 macro(RoundFloat)
 macro(SafePoint)
 macro(SafePointScalarObject)
+#if INCLUDE_SHENANDOAHGC
+macro(ShenandoahCompareAndExchangeP)
+macro(ShenandoahCompareAndExchangeN)
+macro(ShenandoahCompareAndSwapN)
+macro(ShenandoahCompareAndSwapP)
+macro(ShenandoahWeakCompareAndSwapN)
+macro(ShenandoahWeakCompareAndSwapP)
+macro(ShenandoahIUBarrier)
+macro(ShenandoahLoadReferenceBarrier)
+#endif
 macro(SCMemProj)
 macro(SqrtD)
 macro(SqrtF)
@@ -318,6 +334,7 @@ macro(SubVI)
 macro(SubVL)
 macro(SubVF)
 macro(SubVD)
+macro(MulVB)
 macro(MulVS)
 macro(MulVI)
 macro(MulReductionVI)
@@ -331,6 +348,10 @@ macro(FmaVD)
 macro(FmaVF)
 macro(DivVF)
 macro(DivVD)
+macro(AbsVB)
+macro(AbsVS)
+macro(AbsVI)
+macro(AbsVL)
 macro(AbsVF)
 macro(AbsVD)
 macro(NegVF)
