@@ -96,6 +96,10 @@ enum LoopOptsMode {
   LoopOptsSkipSplitIf,
   LoopOptsVerify,
   LoopOptsLastRound
+#if INCLUDE_SHENANDOAHGC
+  ,LoopOptsShenandoahExpand,
+   LoopOptsShenandoahPostExpand
+#endif
 };
 
 typedef unsigned int node_idx_t;
@@ -319,7 +323,7 @@ class Compile : public Phase {
     void set_table_base_offset(int x)  { assert(_table_base_offset == -1 || x == _table_base_offset, "can't change"); _table_base_offset = x; }
     int      table_base_offset() const { assert(_table_base_offset != -1, "not set yet");                      return _table_base_offset; }
 
-    void emit(CodeBuffer& cb);
+    bool emit(CodeBuffer& cb) const;
 
     // Returns the offset of the last entry (the top) of the constant table.
     int  top_offset() const { assert(_constants.top().offset() != -1, "not bound yet"); return _constants.top().offset(); }
@@ -1096,6 +1100,7 @@ class Compile : public Phase {
   void inline_string_calls(bool parse_time);
   void inline_boxing_calls(PhaseIterGVN& igvn);
   bool optimize_loops(int& loop_opts_cnt, PhaseIterGVN& igvn, LoopOptsMode mode);
+  void remove_root_to_sfpts_edges(PhaseIterGVN& igvn);
 
   // Matching, CFG layout, allocation, code generation
   PhaseCFG*         cfg()                       { return _cfg; }
@@ -1372,6 +1377,9 @@ class Compile : public Phase {
   void          set_clone_map(Dict* d);
 
   bool is_compiling_clinit_for(ciKlass* k);
+#ifdef ASSERT
+  bool _type_verify_symmetry;
+#endif
 };
 
 #endif // SHARE_VM_OPTO_COMPILE_HPP
