@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -57,14 +57,16 @@ const char FileSeparator = '/';
 
 // Compute the Perfect Hashing hash code for the supplied UTF-8 string.
 s4 ImageStrings::hash_code(const char* string, s4 seed) {
+    assert(seed > 0 && "invariant");
     // Access bytes as unsigned.
     u1* bytes = (u1*)string;
+    u4 useed = (u4)seed;
     // Compute hash code.
     for (u1 byte = *bytes++; byte; byte = *bytes++) {
-        seed = (seed * HASH_MULTIPLIER) ^ byte;
+        useed = (useed * HASH_MULTIPLIER) ^ byte;
     }
     // Ensure the result is not signed.
-    return seed & 0x7FFFFFFF;
+    return (s4)(useed & 0x7FFFFFFF);
 }
 
 // Match up a string in a perfect hash table.
@@ -176,8 +178,8 @@ const char* ImageModuleData::package_to_module(const char* package_name) {
     // retrieve package location
     ImageLocation location;
     bool found = _image_file->find_location(path, location);
+    delete[] path;
     if (!found) {
-        delete[] path;
         return NULL;
     }
 
@@ -426,7 +428,7 @@ bool ImageFileReader::open() {
 void ImageFileReader::close() {
     // Deallocate the index.
     if (_index_data) {
-        osSupport::unmap_memory((char*)_index_data, _index_size);
+        osSupport::unmap_memory((char*)_index_data, (size_t)map_size());
         _index_data = NULL;
     }
     // Close file.
